"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.HashicorpVaultSigningManager = exports.VaultSigner = void 0;
const tslib_1 = require("tslib");
const types_1 = require("@polkadot/types");
const util_1 = require("@polkadot/util");
const util_crypto_1 = require("@polkadot/util-crypto");
const hashicorp_vault_1 = require("./hashicorp-vault");
class VaultSigner {
    /**
     * @hidden
     */
    constructor(vault, registry) {
        this.vault = vault;
        this.registry = registry;
        this.currentId = -1;
    }
    /**
     * Sign a payload
     */
    signPayload(payload) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            const { registry } = this;
            const { address, signedExtensions, version } = payload;
            const { name, version: keyVersion } = yield this.getVaultKey(address);
            registry.setSignedExtensions(signedExtensions);
            const signablePayload = registry.createType('ExtrinsicPayload', payload, {
                version,
            });
            return this.signData(name, keyVersion, signablePayload.toU8a(true));
        });
    }
    /**
     * Sign raw data
     */
    signRaw(raw) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            const { address, data } = raw;
            const { name, version } = yield this.getVaultKey(address);
            return this.signData(name, version, (0, util_1.hexToU8a)(data));
        });
    }
    /**
     * @hidden
     *
     * Use the Vault to sign raw data and return the signature + update ID
     */
    signData(name, version, data) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            const body = {
                input: Buffer.from(data).toString('base64'),
                // eslint-disable-next-line @typescript-eslint/naming-convention
                key_version: version,
            };
            const signature = yield this.vault.signData(name, body);
            const id = (this.currentId += 1);
            return {
                signature,
                id,
            };
        });
    }
    /**
     * @hidden
     *
     * Get a key from the Vault
     *
     * @param address - SS58 formatted address
     *
     * @throws if there is no key with that address
     */
    getVaultKey(address) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            const payloadPublicKey = (0, util_1.u8aToHex)((0, util_crypto_1.decodeAddress)(address));
            const allKeys = yield this.vault.fetchAllKeys();
            const foundKey = allKeys.find(({ publicKey }) => publicKey === payloadPublicKey);
            if (!foundKey) {
                throw new Error('The signer cannot sign transactions on behalf of the calling Account');
            }
            return foundKey;
        });
    }
}
exports.VaultSigner = VaultSigner;
class HashicorpVaultSigningManager {
    /**
     * Create an instance of the Hashicorp Vault Signing Manager
     *
     * @param args.url - points to where the Vault's transit engine is hosted (usually `<base-url>/v1/transit`)
     * @param args.token - authentication token used for signing
     */
    constructor(args) {
        const { url, token } = args;
        this.vault = new hashicorp_vault_1.HashicorpVault(url, token);
        this.externalSigner = new VaultSigner(this.vault, new types_1.TypeRegistry());
    }
    /**
     * Set the SS58 format in which returned addresses will be encoded
     */
    setSs58Format(ss58Format) {
        this._ss58Format = ss58Format;
    }
    /**
     * Return the addresses of all Accounts in the Hashicorp Vault
     *
     * @throws if called before calling `setSs58Format`. Normally, `setSs58Format` will be called by the SDK when instantiated
     */
    getAccounts() {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            const ss58Format = this.getSs58Format('getAccounts');
            const keys = yield this.vault.fetchAllKeys();
            return keys.map(({ publicKey }) => (0, util_crypto_1.encodeAddress)(publicKey, ss58Format));
        });
    }
    /**
     * Return a signer object that uses the underlying keyring pairs to sign
     */
    getExternalSigner() {
        return this.externalSigner;
    }
    /**
     * Return the information about each key available to sign extrinsics with in the Hashicorp Vault.
     *
     * @throws if called before calling `setSs58Format`. Normally, `setSs58Format` will be called by the SDK when instantiated
     */
    getVaultKeys() {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            const ss58Format = this.getSs58Format('getVaultKeys');
            const keys = yield this.vault.fetchAllKeys();
            return keys.map(key => (Object.assign(Object.assign({}, key), { address: (0, util_crypto_1.encodeAddress)(key.publicKey, ss58Format) })));
        });
    }
    /**
     * @hidden
     *
     * @throws if the SS58 format hasn't been set yet
     */
    getSs58Format(methodName) {
        const { _ss58Format: format } = this;
        if (format === undefined) {
            throw new Error(`Cannot call '${methodName}' before calling 'setSs58Format'. Did you forget to use this Signing Manager to connect with the Polymesh SDK?`);
        }
        return format;
    }
}
exports.HashicorpVaultSigningManager = HashicorpVaultSigningManager;
//# sourceMappingURL=hashicorp-vault-signing-manager.js.map