"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.HashicorpVault = void 0;
const tslib_1 = require("tslib");
const cross_fetch_1 = (0, tslib_1.__importDefault)(require("cross-fetch"));
const lodash_1 = require("lodash");
class HashicorpVault {
    /**
     * @hidden
     */
    constructor(url, token) {
        this.url = url;
        this.headers = {
            'X-Vault-Token': token,
        };
    }
    /**
     * @hidden
     *
     * Assemble URL from separate path strings
     */
    getUrl(...path) {
        return `${this.url}/${path.join('/')}`;
    }
    /**
     * Retrieve all public keys (hex) stored in the Vault with their respective names and versions
     */
    fetchAllKeys() {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            const { headers } = this;
            const response = yield (0, cross_fetch_1.default)(this.getUrl('keys'), {
                method: 'LIST',
                headers,
            });
            yield this.assertResponseOk(response);
            const { data: { keys }, } = yield response.json();
            const allKeys = yield Promise.all(keys.map(name => this.fetchKeysByName(name)));
            return (0, lodash_1.flatten)(allKeys);
        });
    }
    /**
     * Retrieve all the public keys (hex formatted) associated to a name, and their versions
     *
     * @param name - name of the key for which to fetch the public keys
     */
    fetchKeysByName(name) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            const { headers } = this;
            const response = yield (0, cross_fetch_1.default)(this.getUrl('keys', name), {
                method: 'GET',
                headers,
            });
            yield this.assertResponseOk(response);
            const { data: { keys }, } = yield response.json();
            return (0, lodash_1.map)(keys, ({ public_key: publicKey }, version) => {
                const hexKey = Buffer.from(publicKey, 'base64').toString('hex');
                return {
                    publicKey: `0x${hexKey}`,
                    version: Number(version),
                    name,
                };
            });
        });
    }
    /**
     * Sign data on behalf of a key and return the hex signature
     *
     * @param name - name of the key that will sign
     * @param version - version of the named key to use
     * @param data - data that will be signed
     */
    signData(name, data) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            const { headers } = this;
            const response = yield (0, cross_fetch_1.default)(this.getUrl('sign', name), {
                method: 'POST',
                body: JSON.stringify(data),
                headers,
            });
            yield this.assertResponseOk(response);
            const { data: { signature }, } = yield response.json();
            // strip vault prefix
            const strippedSignature = signature.replace(/vault:v\d+:/, '');
            // convert to hex and prepend 0x00 to indicate this is an ed25519 signature
            const hexSignature = Buffer.from(strippedSignature, 'base64').toString('hex');
            return `0x00${hexSignature}`;
        });
    }
    /**
     *
     * @param res
     */
    assertResponseOk(res) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            const { status, statusText } = res;
            if (![200, 204].includes(status)) {
                const { errors = [] } = yield res.json();
                const reasons = errors.length
                    ? `. Reason(s): "${errors.join('", "').replace(/[\n\t*]/g, '')}"`
                    : '';
                throw new Error(`Vault response error: ${status} - ${statusText}${reasons}`);
            }
        });
    }
}
exports.HashicorpVault = HashicorpVault;
//# sourceMappingURL=hashicorp-vault.js.map